#include "IcomMessage.h"
extern const bool bThreePorts;

// constructor
IcomMessage::IcomMessage(void) :
m_Counter(0)
{
};


// Also check to see if each frequency element is a valid BCD
bool
IcomMessage::isFreq() const
{
	if (m_Counter != 10) return false;
	if ( ! (buffer[3] == 3  || !buffer[3] || buffer[3] == 5) ) return false;
	const uint8_t * cp = &buffer[8];
	const uint8_t * end = &buffer[4];
	for ( ; cp >= end ; --cp)
	{
		if ( (*cp & 0x0f) > 9) return false;
		if ( ( (*cp >> 4) & 0x0f) > 9) return false;
	}
	return true;
}

// Also check to see if each frequency element is a valid BCD
bool
IcomMessage::isFreq25() const
{
	if (m_Counter != 11) return false;
	if (buffer[3] != 0x25) return false;
	const uint8_t * cp = &buffer[9];
	const uint8_t * end = &buffer[5];
	for (; cp >= end; --cp)
	{
		if ((*cp & 0x0f) > 9) return false;
		if (((*cp >> 4) & 0x0f) > 9) return false;
	}
	return true;
}

unsigned long 
IcomMessage::getFreq() const
{
	unsigned long ret = 0;
	const uint8_t * cp = &buffer[8];
	const uint8_t * end = &buffer[4];
	for ( ; cp >= end ; --cp)
	{
		ret *= 100;
		ret += static_cast<unsigned long> (bcdByte(*cp));
	}
	return ret;
}

unsigned long
IcomMessage::getFreq25() const
{
	unsigned long ret = 0;
	const uint8_t * cp = &buffer[9];
	const uint8_t * end = &buffer[5];
	for (; cp >= end; --cp)
	{
		ret *= 100;
		ret += static_cast<unsigned long> (bcdByte(*cp));
	}
	return ret;
}

// put a byte into the message,  but first check to see if there is space
bool
IcomMessage::add(const uint8_t x)
{
	if (m_Counter >= icomMessageMaxLength)
	{
		setMTY();
		return false;
	}
	buffer[m_Counter] = x;
	++m_Counter;
	return true;    
}


void
IcomMessage::dump(void) const
{
	if (!bThreePorts)
	{
		for (uint8_t i = 1; i < m_Counter - 1; ++i)
		{
			if (buffer[i] < 16 ) ///  put leading edge 0 in hex
				Serial3.write('0');
			Serial3.print(buffer[i],HEX);
			Serial3.print(" ");
		}
		Serial3.println();
	}
}

void
IcomMessage::dumpSpecial(void) const
{
	if (!bThreePorts)
	{
        Serial3.print(F("==> FE "));
		for (uint8_t i = 0; i < m_Counter; ++i)
		{
			if (buffer[i] < 16 ) ///  put leading edge 0 in hex
				Serial3.write('0');
			Serial3.print(buffer[i],HEX);
			Serial3.print(" ");
		}
		Serial3.println();
	}
}


//  Does sanity checks on a message to see if it follows the rules
bool
IcomMessage::ValidateReceiveMessage(void)
{
	// any valid ICOM message must be at least 5 bytes bytes long
	if (m_Counter < 5) return false;
	// check the range of the source ID
	uint8_t a = getSourceID();
	if ( ! (a < 0xF0 ) ) return false;
	// check the range of the destination ID
	a = getDestinationID();
	if ( ! (a < 0xF0 ) )  return false;
	// check the range of the commands
	a = getCommand();
	if ( ! (a < 0x28 || a == 0xfa || a == 0xfb)) return false;
	// check the length of various commands
	switch (a)
	{
	case 5:  // writes operating frequency
	case 0:  // Transfers operating frequency data
		if (m_Counter != 10) return false;
		break;
	case 1:  // Transfer operating mode data
		if (!( m_Counter == 7 || m_Counter == 6 || m_Counter == 5) ) return false;
		break;
	case 3:  // Read out operating frequency data  COMMON!!
		if (! (m_Counter == 5 || m_Counter == 10)) return false;
		break;
	case 4: // Operating mode and IF passband bandwidth  COMMON!
		if (! (m_Counter == 5 || m_Counter == 7 || m_Counter == 6)) return false;
		break;
	case 6: // Write Operating mode and IF pass bandwidth
		if (!(m_Counter == 7 || m_Counter == 6) ) return false;
		break;
	default:
		break;
	}
	if (a == 4 && m_Counter == 7)
	{
		if (getMode() > 0x14) return false;
		if (getBW() > 6) return false;
	}
	return true;  
}


void
IcomMessage::setSourceDestinationID(const uint8_t source, const uint8_t dest)
{
	buffer[0] = icomMessageStartByte;
	buffer[1] = dest;
	buffer[2] = source;
}

void
IcomMessage::setModeBW(const uint8_t mode, const uint8_t bw)
{
		m_Counter = 7;
		buffer[3] = 4; // Mode BW command
		buffer[4] = mode;
		buffer[5] = bw;
		buffer[6] = icomMessageStopByte;
}

void
IcomMessage::setFreq(const unsigned long freq)
{
	m_Counter = 10;
	buffer[3] = 3; // Frequency mode
	unsigned long f =  (freq);
	for (uint8_t i = 0; i < 5; ++i)
	{
		unsigned long x = f % 100;
		buffer[4 + i] = bcdByteEncode(static_cast<uint8_t>(x));
		f = f / 100;
	}
	buffer[9] = icomMessageStopByte;
}

void
IcomMessage::setFreq25(const unsigned long freq, const uint8_t vfo)
{
	m_Counter = 11;
	buffer[3] = 0x25; // Frequency mode
	buffer[4] = vfo;
	unsigned long f = (freq);
	for (uint8_t i = 0; i < 5; ++i)
	{
		unsigned long x = f % 100;
		buffer[5 + i] = bcdByteEncode(static_cast<uint8_t>(x));
		f = f / 100;
	}
	buffer[10] = icomMessageStopByte;
}
void IcomMessage::setPower(const uint8_t value)
{
    m_Counter = 8;
 //   uint8_t tempValue = value;
    buffer[3] = 0x14;  // set power command
    buffer[4] = 0x0A;
	const uint8_t hundred = 100;
    buffer[5] = bcdByteEncode(value / hundred);
    buffer[6] = bcdByteEncode(value % hundred);
    buffer[7] = icomMessageStopByte;
}

void IcomMessage::setSplitOff()
{
	m_Counter = 6;
	buffer[3] = 0x0f;
	buffer[4] = '\0';
	buffer[5] = icomMessageStopByte;
}


void IcomMessage::setMainReadout()
{
	m_Counter = 6;
	buffer[3] = static_cast<uint8_t>(0x07);
	buffer[4] = static_cast<uint8_t>(0xd0);
	buffer[5] = icomMessageStopByte;
}

// ask the power level
void IcomMessage::setPowerQuery()
{
    m_Counter = 6;
 //   byte tempValue = value;
    buffer[3] = 0x14;  // set power command
    buffer[4] = 0x0A;
    buffer[5] = icomMessageStopByte;
}

bool IcomMessage::isPower()
{
    return (buffer[3] == 0x14 && buffer[4] == 0x0A);
}

uint8_t IcomMessage::getByteResponse(const uint8_t offset)
{
    if (m_Counter < offset + 3)
        return 0;
	uint8_t ret = bcdByte(buffer[offset]) * 100;
    ret += bcdByte(buffer[offset+1]);
    return ret;
}

void 
IcomMessage::setFreqCmd0(const unsigned long freq)
{
	setFreq(freq);
	buffer[3] = 0;  // set command to zero
}

void
IcomMessage::setModeBWCmd1(const uint8_t mode, const uint8_t bw)
{
	setModeBW(mode,bw);
	buffer[3] = 1;  // set command as 1
}

bool
IcomMessage::isEqual(const IcomMessage & x)
{
	if (x.m_Counter != m_Counter) return false;
	for (byte i = 0 ; i < m_Counter ; ++i)
		if (x.buffer[i] != buffer[i]) return false;
	return true;
}

void
IcomMessage::extendlongby(const uint8_t x)
{
	for (uint8_t i = 0; i < x; ++i)
		buffer[i+5] = '\0';
	m_Counter += x;
	buffer[5+x] = icomMessageStopByte;
}

void
IcomMessage::copy(const IcomMessage &a)
{
	m_Counter = a.m_Counter;
	::memcpy(buffer,a.buffer,icomMessageMaxLength);
}
